@@ -34,6 +34,7 @@ do
 
 	$PSQL test <<EOF
 CREATE TABLE t1(i INTEGER);
+CREATE TABLE t2(i INTEGER);
 CREATE FUNCTION f1(INTEGER) returns INTEGER AS 'SELECT \$1' LANGUAGE SQL;
 SELECT * FROM t1;		-- this load balances
 SELECT f1(1);		-- this does not load balance
@@ -79,6 +80,49 @@ EOF
 	fi
 	echo ok: white function list works.
 
+# check if black query pattern list worked
+	./shutdownall
+	echo "black_query_pattern_list = 'SELECT \'a\'\;;SELECT 1\;;SELECT \'\;\'\;;SELECT \* FROM t1\;;^.*t2.*\;$;^.*f1.*$'" >> etc/pgpool.conf
+	./startall
+	wait_for_pgpool_startup
+
+queries=`cat << EOF
+SELECT * FROM t1;
+SELECT 'a';
+SELECT 1;
+SELECT ';';
+SELECT * FROM t2;
+SELECT f1(1);
+EOF
+`
+	echo "$queries" | while read query; do
+		$PSQL test -c "$query"
+
+		# If master-slave mode, all queries are sent to primary node only.
+		# If query match both black_query_pattern_list and white_function_list,
+		# white_function_list will be ignored, and query is sent to primary node only.
+		#
+		# If replication node, all queries are load-blanced.
+		if [[ $mode = "s" ]];then
+			node_id=0
+		else
+			node_id=1
+		fi
+		fgrep "${query}" log/pgpool.log | grep "DB node id: "`echo $node_id` > /dev/null 2>&1
+
+		if [ $? != 0 ];then
+			# expected result not found
+			echo "fail: black query: ${query} is load-blanced."
+			./shutdownall
+			exit 1
+		fi
+	done
+
+	if [ $? -eq 1 ]; then
+		exit 1
+	fi
+	echo ok: black query pattern list works.
+
 # in replication mode if load_balance_mode = off, SELECT query inside
 # an explicit transaction should be sent to master only.
 	if [ $mode = "r" ];then
